@@ -7,8 +7,8 @@
     All generated figures can be accessed via the following link:
     https://github.com/Evolutionary-Intelligence/pypop/tree/main/docs/repeatability/r1es
 
-    Luckily our code could repeat the data reported in the original paper *well*.
-    Therefore, we argue that the repeatability of `R1ES` could be **well-documented**.
+    Luckily our Python code could repeat the data reported in the paper *well*.
+    Therefore, we argue that its repeatability could be **well-documented**.
 """
 import pickle
 
@@ -45,28 +45,25 @@ def plot(function, ndim):
 
 
 class Fig2(R1ES):  # to save data for Fig. 2
-    def optimize(self, fitness_function=None, args=None):
-        fit = ES.optimize(self, fitness_function)
-        xx, mean, p, s, y = self.initialize(args)
-        fit.append(y[0])
+    def optimize(self, fitness_function=None, args=None):  # for all generations (iterations)
+        fitness = ES.optimize(self, fitness_function)
+        x, mean, p, s, y = self.initialize(args)
+        self._print_verbose_info(fitness, y[0])
         e1 = np.hstack((1, np.zeros((self.ndim_problem - 1,))))  # for similarity between p and e1
         f, p_norm, stepsize, theta = [], [], [], []  # for plotting data
-        while True:
+        while not self._check_terminations():
             y_bak = np.copy(y)
-            xx, y = self.iterate(xx, mean, p, y, args)
-            if self.saving_fitness:
-                fit.extend(y)
-            if self._check_terminations():
-                break
-            mean, p, s = self._update_distribution(xx, mean, p, s, y, y_bak)
+            # sample and evaluate offspring population
+            x, y = self.iterate(x, mean, p, y, args)
             self._n_generations += 1
-            self._print_verbose_info(y)
-            xx, mean, p, s, y = self.restart_reinitialize(args, xx, mean, p, s, y, fit)
+            self._print_verbose_info(fitness, y)
+            mean, p, s = self._update_distribution(x, mean, p, s, y, y_bak)
+            x, mean, p, s, y = self.restart_reinitialize(args, x, mean, p, s, y, fitness)
             f.append(np.min(y))
             p_norm.append(np.linalg.norm(p))
             stepsize.append(self.sigma)
-            theta.append(np.arccos(np.abs(np.dot(p, e1)) / p_norm[-1]))
-        res = self._collect_results(fit, mean)
+            theta.append(np.arccos(np.abs(np.dot(p, e1))/p_norm[-1]))
+        res = self._collect(fitness, y, mean)
         res['p'] = p
         res['s'] = s
         res['f'] = np.sqrt(f)  # for plotting data
@@ -82,20 +79,20 @@ if __name__ == '__main__':
     ndim_problem = 200
     problem = {'fitness_function': cigar,
                'ndim_problem': ndim_problem,
-               'lower_boundary': -10 * np.ones((ndim_problem,)),
-               'upper_boundary': 10 * np.ones((ndim_problem,))}
+               'lower_boundary': -10*np.ones((ndim_problem,)),
+               'upper_boundary': 10*np.ones((ndim_problem,))}
     options = {'fitness_threshold': 1e-8,
                'seed_rng': 2022,  # not given in the original paper
-               'sigma': 20 / 3}
+               'sigma': 20/3}
     r1es = Fig2(problem, options)
     results = r1es.optimize()
-    x = np.arange(len(results['f'])) + 1
+    xx = np.arange(len(results['f'])) + 1
     plt.figure()
     plt.yscale('log')
-    plt.plot(x, results['f'], 'b-', label='sqrt(f)', fillstyle='none')
-    plt.plot(x, results['p_norm'], 'g-', label='||p||', fillstyle='none')
-    plt.plot(x, results['stepsize'], 'k-', label='sigma', fillstyle='none')
-    plt.plot(x, results['theta'], 'r-', label='theta', fillstyle='none')
+    plt.plot(xx, results['f'], 'b-', label='sqrt(f)', fillstyle='none')
+    plt.plot(xx, results['p_norm'], 'g-', label='||p||', fillstyle='none')
+    plt.plot(xx, results['stepsize'], 'k-', label='sigma', fillstyle='none')
+    plt.plot(xx, results['theta'], 'r-', label='theta', fillstyle='none')
     plt.xticks(ticks=[1, 1000, 2000, 3000])
     plt.xlim(1, 3000)
     plt.yticks(ticks=[1e-9, 1e-6, 1e-3, 1e0, 1e3, 1e6])
@@ -108,12 +105,12 @@ if __name__ == '__main__':
     for func in [ellipsoid, discus, rosenbrock]:
         problem = {'fitness_function': func,
                    'ndim_problem': ndim_problem,
-                   'lower_boundary': -10 * np.ones((ndim_problem,)),
-                   'upper_boundary': 10 * np.ones((ndim_problem,))}
+                   'lower_boundary': -10*np.ones((ndim_problem,)),
+                   'upper_boundary': 10*np.ones((ndim_problem,))}
         options = {'fitness_threshold': 1e-8,
                    'max_function_evaluations': 1e8,
                    'seed_rng': 2022,  # not given in the original paper
-                   'sigma': 20 / 3,
+                   'sigma': 20.0/3.0,
                    'saving_fitness': 1,
                    'is_restart': False}
         r1es = Fig2(problem, options)
