@@ -6,16 +6,14 @@ from pypop7.optimizers.es.es import ES
 class MAES(ES):
     """Matrix Adaptation Evolution Strategy (MAES).
 
-    .. note:: `MAES` is an interesting *simplified* version of the well-established `CMA-ES` but nearly without
-       significant performance loss, designed carefully by `Beyer <https://homepages.fhv.at/hgb/>`_ and `Sendhoff
+    .. note:: `MAES` is a powerful *simplified* version of the well-established `CMA-ES` algorithm nearly without
+       significant performance loss, designed in 2017 by `Beyer <https://homepages.fhv.at/hgb/>`_ and `Sendhoff
        <https://tinyurl.com/26szwuaa>`_. One obvious advantage of such a simplification is to help better understand
-       the underlying working principles (e.g., **invariance** and **unbias**) of `CMA-ES`, which is often thought to
-       be rather complex for newcomers.
-
-       It is **highly recommended** to first attempt other more advanced ES variants (e.g., `LMCMA`, `LMMAES`) for
-       large-scale black-box optimization, since `MAES` has a *cubic* time complexity w.r.t. each sampling. Note that
-       another improved version called `FMAES` provides a *relatively more efficient* implementation for `MAES` with
-       *quadratic* time complexity w.r.t. each sampling.
+       the underlying working principles (e.g., **invariance** and **unbias**) of `CMA-ES`, which are often thought to
+       be rather complex for newcomers. It is **highly recommended** to first attempt more advanced ES variants (e.g.,
+       `LMCMA`, `LMMAES`) for large-scale black-box optimization, since `MAES` has a *cubic* time complexity (w.r.t.
+       each sampling). Note that another improved version called `FMAES` provides a *relatively more efficient*
+       implementation for `MAES` with *quadratic* time complexity (w.r.t. each sampling).
 
     Parameters
     ----------
@@ -38,13 +36,13 @@ class MAES(ES):
                     bounded by `problem['lower_boundary']` and `problem['upper_boundary']`.
 
                 * 'n_individuals' - number of offspring, aka offspring population size (`int`, default:
-                  `4 + int(3*np.log(self.ndim_problem))`),
+                  `4 + int(3*np.log(problem['ndim_problem']))`),
                 * 'n_parents'     - number of parents, aka parental population size (`int`, default:
-                  `int(self.n_individuals/2)`).
+                  `int(options['n_individuals']/2)`).
 
     Examples
     --------
-    Use the `ES` optimizer `MAES` to minimize the well-known test function
+    Use the optimizer `MAES` to minimize the well-known test function
     `Rosenbrock <http://en.wikipedia.org/wiki/Rosenbrock_function>`_:
 
     .. code-block:: python
@@ -65,7 +63,7 @@ class MAES(ES):
        >>> results = maes.optimize()  # run the optimization process
        >>> # return the number of function evaluations and best-so-far fitness
        >>> print(f"MAES: {results['n_function_evaluations']}, {results['best_so_far_y']}")
-       MAES: 5000, 4.840520170399301e-17
+       MAES: 5000, 2.129367016460251e-19
 
     For its correctness checking of coding, refer to `this code-based repeatability report
     <https://tinyurl.com/3zvve79b>`_ for more details.
@@ -73,13 +71,13 @@ class MAES(ES):
     Attributes
     ----------
     mean          : `array_like`
-                    initial mean of Gaussian search distribution.
+                    initial (starting) point, aka mean of Gaussian search distribution.
     n_individuals : `int`
                     number of offspring, aka offspring population size.
     n_parents     : `int`
                     number of parents, aka parental population size.
     sigma         : `float`
-                    final mutation strength.
+                    final global step-size, aka mutation strength.
 
     References
     ----------
@@ -98,33 +96,33 @@ class MAES(ES):
     IEEE Transactions on Evolutionary Computation, 21(5), pp.746-759.
     https://ieeexplore.ieee.org/document/7875115
 
-    See the official Matlab version from Beyer:
+    See the official Matlab version from Prof. Beyer:
     https://homepages.fhv.at/hgb/downloads/ForDistributionFastMAES.tar
     """
     def __init__(self, problem, options):
         ES.__init__(self, problem, options)
         self.options = options
-        self.c_s = None  # for M10 in Fig. 3
-        self.alpha_cov = 2.0  # for M11 in Fig. 3 (α_cov)
-        self.c_1 = None  # for M11 in Fig. 3
-        self.c_w = None  # for M11 in Fig. 3 (c_μ)
-        self.d_sigma = None  # for M12 in Fig. 3 (d_σ)
-        self._s_1 = None  # for M10 in Fig. 3
-        self._s_2 = None  # for M10 in Fig. 3
+        self.c_s = None
+        self.c_1 = None
+        self.c_w = None
+        self.d_sigma = None
+        self._alpha_cov = 2.0
+        self._s_1 = None
+        self._s_2 = None
         self._fast_version = options.get('_fast_version', False)
         if not self._fast_version:
-            self._diag_one = np.diag(np.ones((self.ndim_problem,)))  # for M11 in Fig. 3
+            self._diag_one = np.eye(self.ndim_problem)
 
     def _set_c_w(self):
-        return np.minimum(1.0 - self.c_1, self.alpha_cov*(self._mu_eff + 1.0/self._mu_eff - 2.0) /
-                          (np.power(self.ndim_problem + 2.0, 2) + self.alpha_cov*self._mu_eff/2.0))
+        return np.minimum(1.0 - self.c_1, self._alpha_cov*(self._mu_eff + 1.0/self._mu_eff - 2.0) /
+                          (np.power(self.ndim_problem + 2.0, 2) + self._alpha_cov*self._mu_eff/2.0))
 
     def _set_d_sigma(self):
         return 1.0 + self.c_s + 2.0*np.maximum(0.0, np.sqrt((self._mu_eff - 1.0)/(self.ndim_problem + 1.0)) - 1.0)
 
-    def initialize(self, is_restart=False):  # for M1 in Fig. 3
+    def initialize(self, is_restart=False):
         self.c_s = self.options.get('c_s', (self._mu_eff + 2.0)/(self._mu_eff + self.ndim_problem + 5.0))
-        self.c_1 = self.options.get('c_1', self.alpha_cov/(np.power(self.ndim_problem + 1.3, 2) + self._mu_eff))
+        self.c_1 = self.options.get('c_1', self._alpha_cov/(np.power(self.ndim_problem + 1.3, 2) + self._mu_eff))
         self.c_w = self.options.get('c_w', self._set_c_w())
         self.d_sigma = self.options.get('d_sigma', self._set_d_sigma())
         self._s_1 = 1.0 - self.c_s
@@ -133,67 +131,67 @@ class MAES(ES):
         d = np.empty((self.n_individuals, self.ndim_problem))  # search directions
         mean = self._initialize_mean(is_restart)  # mean of Gaussian search distribution
         s = np.zeros((self.ndim_problem,))  # evolution path
-        tm = np.diag(np.ones((self.ndim_problem,)))  # transformation matrix M
+        tm = np.eye(self.ndim_problem)  # transformation matrix
         y = np.empty((self.n_individuals,))  # fitness (no evaluation)
+        self._list_initial_mean.append(np.copy(mean))
         return z, d, mean, s, tm, y
 
     def iterate(self, z=None, d=None, mean=None, tm=None, y=None, args=None):
-        for k in range(self.n_individuals):  # for M3 in Fig. 3 (sample offspring population)
+        for k in range(self.n_individuals):  # to sample offspring population
             if self._check_terminations():
                 return z, d, y
-            z[k] = self.rng_optimization.standard_normal((self.ndim_problem,))  # for M4 in Fig. 3
-            d[k] = np.squeeze(np.dot(tm, z[k][:, np.newaxis]))  # for M5 in Fig. 3
-            y[k] = self._evaluate_fitness(mean + self.sigma*d[k], args)  # for M6 in Fig. 3
+            z[k] = self.rng_optimization.standard_normal((self.ndim_problem,))
+            d[k] = np.dot(tm, z[k])
+            y[k] = self._evaluate_fitness(mean + self.sigma*d[k], args)
         return z, d, y
 
     def _update_distribution(self, z=None, d=None, mean=None, s=None, tm=None, y=None):
-        order = np.argsort(y)  # for M8 in Fig. 3
-        # set for M9, M10, M11 in Fig. 3
+        order = np.argsort(y)
         d_w, z_w, zz_w = np.zeros((self.ndim_problem,)), np.zeros((self.ndim_problem,)), None
         if not self._fast_version:
-            zz_w = np.zeros((self.ndim_problem, self.ndim_problem))  # for M11 in Fig. 3
+            zz_w = np.zeros((self.ndim_problem, self.ndim_problem))
         for k in range(self.n_parents):
             d_w += self._w[k]*d[order[k]]
             z_w += self._w[k]*z[order[k]]
             if not self._fast_version:
-                zz_w += self._w[k]*np.dot(z[order[k]][:, np.newaxis], z[order[k]][np.newaxis, :])
-        # update distribution mean (for M9 in Fig. 3)
+                zz_w += self._w[k]*np.outer(z[order[k]], z[order[k]])
+        # update distribution mean
         mean += self.sigma*d_w
         # update evolution path (s) and transformation matrix (M)
-        s = self._s_1*s + self._s_2*z_w  # for M10 in Fig. 3
+        s = self._s_1*s + self._s_2*z_w
         if not self._fast_version:
-            tm_1 = self.c_1*(np.dot(s[:, np.newaxis], s[np.newaxis, :]) - self._diag_one)
+            tm_1 = self.c_1*(np.outer(s, s) - self._diag_one)
             tm_2 = self.c_w*(zz_w - self._diag_one)
-            tm += 0.5*np.dot(tm, tm_1 + tm_2)  # for M11 in Fig. 3
+            tm += 0.5*np.dot(tm, tm_1 + tm_2)
         else:
             tm = (1.0 - 0.5*(self.c_1 + self.c_w))*tm + (0.5*self.c_1)*np.dot(
                 np.dot(tm, s[:, np.newaxis]), s[np.newaxis, :])
             for k in range(self.n_parents):
-                tm += (0.5*self.c_w)*self._w[k]*np.dot(d[order[k]][:, np.newaxis], z[order[k]][np.newaxis, :])
-        # update global step-size (for M12 in Fig. 3)
+                tm += (0.5*self.c_w)*self._w[k]*np.outer(d[order[k]], z[order[k]])
+        # update global step-size
         self.sigma *= np.exp(self.c_s/self.d_sigma*(np.linalg.norm(s)/self._e_chi - 1.0))
         return mean, s, tm
 
     def restart_reinitialize(self, z=None, d=None, mean=None, s=None, tm=None, y=None):
-        if ES.restart_reinitialize(self):
+        if ES.restart_reinitialize(self, y):
             z, d, mean, s, tm, y = self.initialize(True)
         return z, d, mean, s, tm, y
 
     def optimize(self, fitness_function=None, args=None):  # for all generations (iterations)
         fitness = ES.optimize(self, fitness_function)
         z, d, mean, s, tm, y = self.initialize()
-        while True:
+        while not self.termination_signal:
             # sample and evaluate offspring population
             z, d, y = self.iterate(z, d, mean, tm, y, args)
-            if self.saving_fitness:
-                fitness.extend(y)
             if self._check_terminations():
                 break
             mean, s, tm = self._update_distribution(z, d, mean, s, tm, y)
-            self._print_verbose_info(y)
+            self._print_verbose_info(fitness, y)
             self._n_generations += 1
             if self.is_restart:
                 z, d, mean, s, tm, y = self.restart_reinitialize(z, d, mean, s, tm, y)
-        results = self._collect_results(fitness, mean)
+        results = self._collect(fitness, y, mean)
         results['s'] = s
+        # by default, do NOT save transformation matrix of search distribution in order to save memory,
+        # owing to its *quadratic* space complexity
         return results
