@@ -1,7 +1,8 @@
 """This code only tests whether the corresponding optimizer could run or not.
   In other words, it *cannot* validate the correctness regarding its working procedure.
-  For the correctness validation of programming, refer to the following link:
-    https://github.com/Evolutionary-Intelligence/pypop/wiki/Correctness-Validation-of-Optimizers.md
+
+  For its correctness checking of coding, the *code-based repeatability report* cannot be provided owing to
+    the lack of some details of its experiments in the original paper.
 """
 import unittest
 import time
@@ -20,16 +21,14 @@ class TestNSA(unittest.TestCase):
             print('*' * 7 + ' ' + f.__name__ + ' ' + '*' * 7)
             problem = {'fitness_function': f,
                        'ndim_problem': ndim_problem,
-                       'lower_boundary': -5 * np.ones((ndim_problem,)),
-                       'upper_boundary': 5 * np.ones((ndim_problem,))}
+                       'lower_boundary': -5*np.ones((ndim_problem,)),
+                       'upper_boundary': 5*np.ones((ndim_problem,))}
             options = {'max_function_evaluations': 2e6,
                        'fitness_threshold': 1e-10,
                        'seed_rng': 0,
-                       'x': 4 * np.ones((ndim_problem,)),
+                       'x': 4*np.ones((ndim_problem,)),
                        'sigma': 0.1,
                        'temperature': 100,
-                       'is_noisy': False,
-                       'n_samples': 1,  # for static optimization
                        'verbose': 20000,
                        'saving_fitness': 200000}
             solver = Solver(problem, options)
